// test program for Class Foo

#include "Foo.h"

static Foo static_foo = Foo( "static_foo2"); 

Foo f()
{
    Foo x;
    return x;
}

void g()
{
    Foo other_foo1 = Foo( "other_foo1"), other_foo2 = Foo( "other_foo2");
    other_foo2 = other_foo1;
}
